<?php

namespace Drp\LaravelSetup\Database;

use Illuminate\Console\Command;

class ConnectionSettings
{
    /** @var \Illuminate\Console\Command */
    private $output;

    /** @var string */
    private $defaultDbName;

    public function __construct(Command $output, string $defaultDbName)
    {
        $this->output = $output;
        $this->defaultDbName = $defaultDbName;
    }

    public static function request(Command $output, string $defaultDbName): array
    {
        return (new self($output, $defaultDbName))->getConnectionSettings();
    }

    private function getConnectionSettings(): array
    {
        $config = $this->askConnectionSettings();
        if ($config['database'] === null || $config['username'] === null || $config['password'] === null) {
            $this->output->error(
                'You must answer all the questions.' .
                PHP_EOL
            );

            return $this->getConnectionSettings();
        }

        return $config;
    }

    private function askConnectionSettings(): array
    {
        return $this->makeConfig(
            $this->output->ask('What is the database host?', '127.0.0.1'),
            $this->output->ask('What is the database port?', '3306'),
            $this->output->ask('What is the database name?', $this->sanitizeDatabaseName($this->defaultDbName)),
            $this->output->ask('What is the database username?', 'root'),
            $this->output->ask('What is the database password?', '')
        );
    }

    private function makeConfig(
        ?string $host,
        ?string $port,
        ?string $database,
        ?string $username,
        ?string $password
    ): array {
        return [
            'driver' => 'mysql',
            'host' => $host,
            'port' => $port,
            'database' => $this->sanitizeDatabaseName($database),
            'username' => $username,
            'password' => $password,
            'unix_socket' => '',
            'charset' => 'utf8',
            'collation' => 'utf8_unicode_ci',
            'prefix' => '',
            'strict' => true,
            'engine' => null,
        ];
    }

    /**
     * @return string|string[]|null
     */
    private function sanitizeDatabaseName(?string $database)
    {
        return preg_replace('/(?![\w_-])./', '', $database);
    }
}
