<?php

namespace Drp\LaravelSetup\Database\Env;

use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class Writer
{
    /**
     * @var array
     */
    private $envMapping;

    /**
     * @var string
     */
    private $envFile;

    public function __construct(array $envMapping, string $envFile = '.env')
    {
        $this->envMapping = $envMapping;
        $this->envFile = $envFile;
    }

    public function getEnvFile(): string
    {
        return $this->envFile;
    }

    public function put(array $config): void
    {
        $envFile = Storage::disk('laravel-setup:root')->get($this->envFile);

        $envFile = preg_replace(
            [
                "/{$this->envMapping['dbHost']}=.*/",
                "/{$this->envMapping['dbPort']}=.*/",
                "/{$this->envMapping['dbName']}=.*/",
                "/{$this->envMapping['dbUsername']}=.*/",
                "/{$this->envMapping['dbPassword']}=.*/",
            ],
            [
                $this->envMapping['dbHost'] . '=' . $config['host'],
                $this->envMapping['dbPort'] . '=' . $config['port'],
                $this->envMapping['dbName'] . '=' . $this->wrapIfSpaces($config['database']),
                $this->envMapping['dbUsername'] . '=' . $this->wrapIfSpaces($config['username']),
                $this->envMapping['dbPassword'] . '=' . $this->wrapIfSpaces($config['password']),
            ],
            $envFile
        );

        Storage::disk('laravel-setup:root')->put($this->envFile, $envFile);
    }

    private function wrapIfSpaces(string $value): string
    {
        if (Str::contains($value, ' ')) {
            return '"' . $value . '"';
        }

        return $value;
    }
}
